@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2016-2022 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,11 +24,10 @@
 
 import itertools
 import os
-import sys
+from typing import Any, Callable
 
 from joblib import Parallel, delayed
 
-
 def CPUThreadCount(enable=True):
   from .Common import globalParameters
   if not enable:
@@ -40,7 +39,7 @@ def CPUThreadCount(enable=True):
       cpu_count = len(os.sched_getaffinity(0))
     cpuThreads = globalParameters["CpuThreads"]
     if cpuThreads < 1:
-        return cpu_count
+        return min(cpu_count, 64) # Max build threads to avoid out-of-memory
     return min(cpu_count, cpuThreads)
 
 def OverwriteGlobalParameters(newGlobalParameters):
@@ -56,36 +55,40 @@ def pcallWithGlobalParamsSingleArg(f, arg, newGlobalParameters):
   OverwriteGlobalParameters(newGlobalParameters)
   return f(arg)
 
-def ParallelMap(function, objects, message="", enable=True, multiArg=True):
-  """
-  Generally equivalent to list(map(function, objects)), possibly executing in parallel.
+def ParallelMap(function: Callable, objects: Any, message: str="", enable: bool=True, multiArg: bool=True):
+    """Executes a function over a list of objects in parallel or sequentially.
 
-    message: A message describing the operation to be performed.
-    enable: May be set to false to disable parallelism.
-    multiArg: True if objects represent multiple arguments
-                (differentiates multi args vs single collection arg)
-  """
-  from .Common import globalParameters
-  from . import Utils
-  threadCount = CPUThreadCount(enable)
-  
-  if threadCount <= 1 and globalParameters["ShowProgressBar"]:
-    # Provide a progress bar for single-threaded operation.
-    return list(map(lambda objs: function(*objs), Utils.tqdm(objects, message)))
-  
-  countMessage = ""
-  try:
-    countMessage = " for {} tasks".format(len(objects))
-  except TypeError: pass
+    This function is generally equivalent to ``list(map(function, objects))``. However, it provides
+    additional functionality to run in parallel, depending on the 'enable' flag and available CPU
+    threads.
 
-  if message != "": message += ": "
-  print("{0}Launching {1} threads{2}...".format(message, threadCount, countMessage))
-  sys.stdout.flush()
-  
-  pcall = pcallWithGlobalParamsMultiArg if multiArg else pcallWithGlobalParamsSingleArg
-  pargs = zip(objects, itertools.repeat(globalParameters))
-  rv = Parallel(n_jobs=threadCount)(delayed(pcall)(function, a, params) for a, params in pargs)
-  
-  print("{0}Done.".format(message))
-  sys.stdout.flush()
-  return rv
\ No newline at end of file
+    Args:
+        function: The function to apply to each item in 'objects'. If 'multiArg' is True, 'function'
+                  should accept multiple arguments.
+        objects: An iterable of objects to be processed by 'function'. If 'multiArg' is True, each
+                 item in 'objects' should be an iterable of arguments for 'function'.
+        message: Optional; a message describing the operation. Default is an empty string.
+        enable: Optional; if False, disables parallel execution and runs sequentially. Default is True.
+        multiArg: Optional; if True, treats each item in 'objects' as multiple arguments for
+                  'function'. Default is True.
+        verbose: Optional; verbosity level for parallel execution. Default is 0.
+
+    Returns:
+        A list containing the results of applying **function** to each item in **objects**.
+    """
+    from .Common import globalParameters
+    from . import Utils
+    threadCount = CPUThreadCount(enable)
+    
+    if threadCount <= 1:
+      return list(map(lambda objs: function(*objs), Utils.tqdm(objects, desc=message)))
+        
+    inputs = list(zip(objects, itertools.repeat(globalParameters)))
+    message += f": {threadCount} threads, {len(inputs)} tasks"
+
+    pargs = Utils.tqdm(inputs, desc=message)
+    pcall = pcallWithGlobalParamsMultiArg if multiArg else pcallWithGlobalParamsSingleArg
+
+    rv = Parallel(n_jobs=threadCount)(delayed(pcall)(function, a, params) for a, params in pargs)
+    
+    return rv
